#include "utils.h"
#include <map>
#include "stb_image.h"
#include <windows.h>
#include <filesystem>
#include <string>
#include <unordered_map>
#include <mutex>

std::unordered_map<std::string, ImageCacheEntry> textureCache;
std::unordered_map<std::string, ImageCacheEntry> engineTextureCache;



GLuint LoadTextureIfNeeded(const std::string &path)
{
    auto [it, inserted] = textureCache.try_emplace(path, ImageCacheEntry{});

    if (!inserted)
    {
        return it->second.textureID;
    }

    int w, h, channels;
    stbi_set_flip_vertically_on_load(false);
    unsigned char *data = stbi_load(path.c_str(), &w, &h, &channels, 4);
    if (!data)
        return 0;

    GLuint id;
    glGenTextures(1, &id);
    glBindTexture(GL_TEXTURE_2D, id);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    stbi_image_free(data);

    glm::vec2 imageSize(w, h);
    
    it->second = { id, imageSize };

    Logger::LogVerbose("Loaded Image: '%s'", path.c_str());
    return id;
}

GLuint EngineLoadTextureIfNeeded(const std::string &path)
{
    auto [it, inserted] = engineTextureCache.try_emplace(path, ImageCacheEntry{});

    if (!inserted)
    {
        return it->second.textureID;
    }

    int w, h, channels;
    stbi_set_flip_vertically_on_load(false);
    unsigned char *data = stbi_load(path.c_str(), &w, &h, &channels, 4);
    if (!data)
        return 0;

    GLuint id;
    glGenTextures(1, &id);
    glBindTexture(GL_TEXTURE_2D, id);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    stbi_image_free(data);

    glm::vec2 imageSize(w, h);
    
    it->second = { id, imageSize };

    Logger::LogVerbose("Loaded Image: '%s'", path.c_str());
    return id;
}


std::string GetFilenameFromPath(const std::string &path)
{
    static std::unordered_map<std::string, std::string> cache;
    static std::mutex cacheMutex;

    std::scoped_lock lock(cacheMutex);

    auto it = cache.find(path);
    if (it != cache.end())
        return it->second;

    // Extract filename
    size_t pos1 = path.find_last_of("/\\");
    std::string filename = (pos1 == std::string::npos) ? path : path.substr(pos1 + 1);

    cache[path] = filename;
    return filename;
}

bool DeleteLatestLogFile()
{
    // Get %USERPROFILE%
    char userPath[MAX_PATH];
    DWORD len = GetEnvironmentVariableA("USERPROFILE", userPath, MAX_PATH);
    if (len == 0 || len >= MAX_PATH)
        return false;

    std::filesystem::path logPath = std::filesystem::path(userPath) / ".CreateEngine" / ".latest.log";

    if (std::filesystem::exists(logPath))
    {
        std::error_code ec;
        std::filesystem::remove(logPath, ec);
        return !ec; // true = success
    }

    return false; // File didn't exist
}