#include "Inspector.h"

#include "../../Entitys/Object.h"
#include "../../components/SpriteComponent.h"
#include "../../components/CameraComponent.h"
#include "../../components/LightComponent.h"
#include "../../components/TilemapComponent.h"
#include "../../components/ScriptComponent.h"
#include "../../components/PhysicsComponent.h"

void DrawInspectorUI(std::shared_ptr<Object> selected)
{
    PROFILE_ENGINE_SCOPE("Engine::DrawInspectorUI");
    ImGui::Begin("Inspector");

    if (!selected)
    {
        ImGui::TextDisabled("No object selected.");
        ImGui::End();
        return;
    }

    // --- keep track of the name buffer ---
    static char nameBuffer[128] = "";
    static std::shared_ptr<Object> lastSelected = nullptr;
    if (selected != lastSelected)
    {
        strncpy(nameBuffer, selected->GetName().c_str(), sizeof(nameBuffer));
        nameBuffer[sizeof(nameBuffer) - 1] = '\0';
        lastSelected = selected;
    }

    // --- General ---
    ImGui::TextUnformatted("General");
    ImGui::Separator();

    // Name edit
    if (ImGui::InputText("Name", nameBuffer, sizeof(nameBuffer)))
        selected->SetName(nameBuffer);

    // Visibility toggle
    bool visible = selected->GetVisable();
    if (ImGui::Checkbox("Visible", &visible))
        selected->SetVisable(visible);

    // --- Transform ---
    ImGui::TextUnformatted("Transform");
    ImGui::Separator();

    glm::vec2 pos = selected->GetLocalPosition();
    if (ImGui::DragFloat2("Position", &pos.x, 0.1f))
        selected->SetLocalPosition(pos);

    float rotation = selected->GetLocalRotation();
    if (ImGui::DragFloat("Rotation", &rotation, 1.0f))
        selected->SetLocalRotation(rotation);

    int layer = selected->layer;
    if (ImGui::InputInt("Layer", &layer))
        selected->layer = layer;

    ImGui::Text("ID: %d", selected->uid.id);
    ImGui::Text("UID: %s", selected->uid.uuid.c_str());

    ImGui::Spacing();
    ImGui::SeparatorText("Add Component");

    static const char *componentTypes[] = {
        "SpriteComponent",
        "CameraComponent",
        "LightComponent",
        "ScriptComponent",
        "TilemapComponent"};

    static int selectedIndex = -1;

    if (ImGui::BeginCombo("##ComponentCombo", selectedIndex >= 0 ? componentTypes[selectedIndex] : "Select Component"))
    {
        for (int i = 0; i < IM_ARRAYSIZE(componentTypes); ++i)
        {
            bool isSelected = (selectedIndex == i);
            if (ImGui::Selectable(componentTypes[i], isSelected))
                selectedIndex = i;
            if (isSelected)
                ImGui::SetItemDefaultFocus();
        }
        ImGui::EndCombo();
    }

    ImGui::SameLine();
    if (ImGui::Button("Add") && selectedIndex != -1 && selected)
    {
        const std::string type = componentTypes[selectedIndex];

        if (type == "SpriteComponent" && !selected->GetComponent<SpriteComponent>())
            selected->AddComponent<SpriteComponent>();
        else if (type == "CameraComponent" && !selected->GetComponent<CameraComponent>())
            selected->AddComponent<CameraComponent>();
        else if (type == "LightComponent" && !selected->GetComponent<LightComponent>())
            selected->AddComponent<LightComponent>();
        else if (type == "ScriptComponent" && !selected->GetComponent<ScriptComponent>())
            selected->AddComponent<ScriptComponent>();
        else if (type == "TilemapComponent" && !selected->GetComponent<TilemapComponent>())
            selected->AddComponent<TilemapComponent>();
    }

    if (auto sprite = selected->GetComponent<SpriteComponent>())
    {
        ImGui::SeparatorText("Sprite Component");

        // — Render Type (unchanged) —
        const char *renderTypes[] = {"Unlit", "Lit"};
        int currentTypeIndex = static_cast<int>(sprite->GetRenderType());
        if (ImGui::Combo("Render Type", &currentTypeIndex, renderTypes, IM_ARRAYSIZE(renderTypes)))
            sprite->SetRenderType(static_cast<SpriteComponent::RenderType>(currentTypeIndex));

        ImGui::SeparatorText("Texture");

        // — Texture File with Drag & Drop + Load Button —
        {
            std::string texFile = GetFilenameFromPath(sprite->GetTexturePath());

            // filename text
            ImGui::Text(texFile.c_str());
            // accept drag‑and‑drop here
            if (ImGui::BeginDragDropTarget())
            {
                if (const ImGuiPayload *payload = ImGui::AcceptDragDropPayload("ASSET_TEXTURE"))
                {
                    const char *dropped = (const char *)payload->Data;
                    sprite->SetTexture(dropped);
                }
                ImGui::EndDragDropTarget();
            }

            if (ImGui::SmallButton("Load##SpriteTex"))
            {
                auto path = OpenFileDialog(FileDialogType::Images);
                if (!path.empty())
                    sprite->SetTexture(path);
            }

        }
        ImGui::SeparatorText("Normal Map");
        {
            std::string normFile = GetFilenameFromPath(sprite->GetNormalMapPath());

            ImGui::Text(normFile.c_str());
            if (ImGui::BeginDragDropTarget())
            {
                if (const ImGuiPayload *payload = ImGui::AcceptDragDropPayload("ASSET_TEXTURE"))
                {
                    const char *dropped = (const char *)payload->Data;
                    sprite->SetNormalMap(dropped);
                }
                ImGui::EndDragDropTarget();
            }
            if (ImGui::SmallButton("Load##SpriteNorm"))
            {
                auto path = OpenFileDialog(FileDialogType::Images);
                if (!path.empty())
                    sprite->SetNormalMap(path);
            }

        }

        ImGui::SeparatorText("Info");


        // — Size Display & Remove —
        glm::vec2 size = sprite->GetSize();
        ImGui::Text("Size: %.0f × %.0f", size.x, size.y);

        ImGui::Separator();

        if (ImGui::Button("Remove SpriteComponent"))
            selected->RemoveComponent<SpriteComponent>();
    }

    if (auto cam = selected->GetComponent<CameraComponent>())
    {
        ImGui::SeparatorText("Camera Component");

        float fov = cam->GetFOV();
        float aspect = cam->GetAspect();
        float zoom = cam->GetZoom();

        if (ImGui::DragFloat("FOV", &fov, 0.1f, 1.0f, 179.0f))
            cam->SetFOV(fov);
        if (ImGui::DragFloat("Aspect", &aspect, 0.01f, 0.1f, 5.0f))
            cam->SetAspect(aspect);
        if (ImGui::DragFloat("Zoom", &zoom, 0.1f, 0.1f, 10.0f))
            cam->SetZoom(zoom);

        bool isPrimary = cam->IsPrimary();
        if (ImGui::Checkbox("Primary", &isPrimary))
            cam->SetPrimary(isPrimary);

        if (ImGui::Button("Remove CameraComponent"))
            selected->RemoveComponent<CameraComponent>();
    }

    if (auto light = selected->GetComponent<LightComponent>())
    {
        ImGui::SeparatorText("Light Component");

        glm::vec3 color = light->GetColor();
        float intensity = light->GetIntensity();
        float radius = light->GetRadius();
        float falloff = light->GetFalloff();
        int type = light->GetType();

        if (ImGui::ColorEdit3("Color", &color.x))
            light->SetColor(color);
        if (ImGui::DragFloat("Intensity", &intensity, 0.05f, 0.0f, 10.0f))
            light->SetIntensity(intensity);
        if (ImGui::DragFloat("Radius", &radius, 1.0f, 10.0f, 1000.0f))
            light->SetRadius(radius);
        if (ImGui::DragFloat("Falloff", &falloff, 0.05f, 0.1f, 5.0f))
            light->SetFalloff(falloff);

        // const char* types[] = { "Point", "Directional" };
        // if (ImGui::Combo("Type", &type, types, IM_ARRAYSIZE(types)))
        //     light->SetType(type);

        if (ImGui::Button("Remove LightComponent"))
            selected->RemoveComponent<LightComponent>();
    }
    if (auto script = selected->GetComponent<ScriptComponent>())
    {
        ImGui::SeparatorText("Script Component");

        std::string path = script->GetScriptPath();
        ImGui::Text("Script Path:");
        ImGui::TextWrapped("%s", path.c_str());

        if (ImGui::Button("Load Script"))
        {
            std::string chosen = OpenFileDialog(FileDialogType::Scripts); // Add 'Scripts' filter if needed
            if (!chosen.empty())
                script->SetScriptPath(chosen);
        }

        ImGui::SameLine();
        if (ImGui::Button("Reload"))
        {
            script->ReloadScript();
        }

        if (ImGui::Button("Remove ScriptComponent"))
            selected->RemoveComponent<ScriptComponent>();
    }
    if (auto phys = selected->GetComponent<PhysicsComponent>())
    {
        ImGui::SeparatorText("Physics Component");

        glm::vec2 vel = phys->GetVelocity();
        if (ImGui::DragFloat2("Velocity", &vel.x))
            phys->SetVelocity(vel);

        float friction = phys->GetFriction();
        if (ImGui::DragFloat("Friction", &friction, 0.01f, 0.0f, 1.0f))
            phys->SetFriction(friction);

        bool fixed = phys->IsFixedRotation();
        if (ImGui::Checkbox("Fixed Rotation", &fixed))
            phys->SetFixedRotation(fixed);

        if (ImGui::Button("Remove PhysicsComponent"))
            selected->RemoveComponent<PhysicsComponent>();
    }

    if (auto tilemap = selected->GetComponent<TilemapComponent>())
    {
        ImGui::SeparatorText("Tilemap Component");

        ImGui::Text("Refer to Tilemap Editor Window");

        if (ImGui::Button("Remove TilemapComponent"))
            selected->RemoveComponent<TilemapComponent>();
    }

    ImGui::End();

    if (auto tilemap = selected->GetComponent<TilemapComponent>())
    {
        tilemap->DrawEditorUI();
    }
}